@@ -5,14 +5,14 @@
  *
  *      Description:  Example how to use the C/C++ Marker API
  *
- *      Version:   5.1.0
- *      Released:  20.11.2020
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Authors:  Thomas Gruber (tr), thomas.roehl@googlemail.com
  *                Riley Weber, rileyw13@protonmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -34,7 +34,7 @@
 #include <unistd.h>
 
 #include <omp.h>
-#include <likwid.h>
+#include <likwid-marker.h>
 
 #ifdef LIKWID_PERFMON
 #define MAX_NUM_EVENTS 10
