@@ -8,13 +8,13 @@
  *                    that is used by likwid to avoid reading the systems architecture at
  *                    each start.
  *
- *      Version:   5.1.0
- *      Released:  20.11.2020
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -97,7 +97,7 @@ end
 print_stdout(string.format("Writing new topology file %s", filename))
 cpuinfo["clock"] = likwid.getCpuClock()
 
-local threadPool_order = {"threadId", "coreId", "packageId", "apicId"}
+local threadPool_order = {"threadId", "coreId", "packageId", "apicId", "dieId"}
 local cacheLevels_order = {"type", "associativity", "sets", "lineSize", "size", "threads", "inclusive"}
 
 for field, value in pairs(cpuinfo) do
