@@ -7,13 +7,13 @@
  *      Description:  An application to use the timeline mode of likwid-perfctr to generate
  *                    realtime plots using feedGnuplot
  *
- *      Version:   5.1.0
- *      Released:  20.11.2020
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -478,7 +478,9 @@ for i, group in pairs(group_list) do
         table.insert(extracmds, string.format("set ylabel %q font \",12\"", group["ytitle"]))
     end
     if group["y2title"] ~= nil then
-        gnucmd = gnucmd .. string.format(" --y2 %d", group["y2funcindex"])
+        for c,cpu in pairs(cpulist) do
+            gnucmd = gnucmd .. string.format(" --y2 %d", (c-1) + (#cpulist * group["y2funcindex"]))
+        end
         table.insert(extracmds, string.format("set y2label %q font \",12\"", group["y2title"]))
     end
     if group["formulas"] then
@@ -488,8 +490,8 @@ for i, group in pairs(group_list) do
             end
         else
             local curveID = 0
-            for c,cpu in pairs(cpulist) do
-                for f, fdesc in pairs(group["formulas"]) do
+            for f, fdesc in pairs(group["formulas"]) do
+                for c,cpu in pairs(cpulist) do
                     gnucmd = gnucmd .. string.format(" --legend %d %q", curveID, "C"..cpu..": "..fdesc["name"])
                     curveID = curveID + 1
                 end
