@@ -5,13 +5,13 @@
  *
  *      Description:  Lua LIKWID interface library
  *
- *      Version:   5.1.0
- *      Released:  20.11.2020
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -154,6 +154,7 @@ likwid.markerRegionCount = likwid_markerRegionCount
 likwid.markerRegionResult = likwid_markerRegionResult
 likwid.markerRegionMetric = likwid_markerRegionMetric
 likwid.initFreq = likwid_initFreq
+likwid.getCpuClockBase = likwid_getCpuClockBase
 likwid.getCpuClockCurrent = likwid_getCpuClockCurrent
 likwid.getCpuClockMin = likwid_getCpuClockMin
 likwid.getConfCpuClockMin = likwid_getConfCpuClockMin
@@ -1559,4 +1560,59 @@ end
 
 likwid.getNvMarkerResults = getNvMarkerResults
 
+local function printTextTable(header, line, print_header)
+    local linelength = 80
+    local headerlength = 0
+    local out = {}
+    local headcount = 0
+    for _, h in pairs(header) do
+        headerlength = headerlength + string.len(h)
+        headcount = headcount + 1
+    end
+    hspaces = math.floor((linelength - headerlength)/(headcount-1))
+    if print_header and print_header == true then
+        table.insert(out, table.concat(header, string.rep(" ",hspaces)))
+    end
+    ltab = {}
+    for i, l in pairs(line) do
+        hlength = string.len(header[i]) + hspaces
+        v = l .. string.rep(" ",hlength - string.len(l))
+        table.insert(ltab, v)
+    end
+    table.insert(out, table.concat(ltab, ""))
+    return table.concat(out, "\n")
+end
+
+likwid.printTextTable = printTextTable
+
+local function perfctr_checkpid(pid)
+    local fname = string.format("/proc/%d/status", pid)
+    local f = io.open(fname, "r")
+    if f ~= nil then
+        f:close()
+        return true
+    end
+    return false
+end
+likwid.perfctr_checkpid = perfctr_checkpid
+
+local function perfctr_pid_cpulist(pid)
+    local cpulist = nil
+    local fname = string.format("/proc/%d/status", pid)
+    local f = io.open(fname, "r")
+    if f ~= nil then
+        local l = f:read("*line")
+        while l do
+            if l:match("Cpus_allowed_list:.*") then
+                cpulist = l:match("Cpus_allowed_list:%s+([0-9%-,]+)")
+                break
+            end
+            l = f:read("*line")
+        end
+        f:close()
+    end
+    return cpulist
+end
+likwid.perfctr_pid_cpulist = perfctr_pid_cpulist
+
 return likwid
