@@ -5,13 +5,13 @@
  *
  *      Description:  Parser for CPU selection strings
  *
- *      Version:   5.1.0
- *      Released:  20.11.2020
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -65,16 +65,27 @@ cpulist_sort(int* incpus, int* outcpus, int length)
     {
         return -1;
     }
-    int inner_loop = ceil((double)length/cpuid_topology->numThreadsPerCore);
-    for (int off = 0; off < cpuid_topology->numThreadsPerCore; off++)
+    for (int k = 0; k < cpuid_topology->numThreadsPerCore; k++)
     {
-        for (int i = 0; i < inner_loop; i++)
+        for (int i = 0; i < length; i++)
         {
-            outcpus[insert] = incpus[(i*cpuid_topology->numThreadsPerCore)+off];
-            insert++;
+            int idx = -1;
+            for (int j = 0; j < cpuid_topology->numHWThreads; j++)
+            {
+                if (cpuid_topology->threadPool[j].apicId == incpus[i])
+                {
+                    idx = j;
+                    break;
+                }
+            }
+            if (idx >= 0 && cpuid_topology->threadPool[idx].threadId == k)
+            {
+                outcpus[insert] = incpus[i];
+                insert++;
+            }
+            if (insert == length) break;
         }
-        if (insert == length)
-            break;
+        if (insert == length) break;
     }
     return insert;
 }
@@ -864,7 +875,8 @@ cpustr_to_cpulist(const char* cpustring, int* cpulist, int length)
         else if ((bstrchrp(strlist->entry[i], 'N', 0) == 0) ||
                  (bstrchrp(strlist->entry[i], 'S', 0) == 0) ||
                  (bstrchrp(strlist->entry[i], 'C', 0) == 0) ||
-                 (bstrchrp(strlist->entry[i], 'M', 0) == 0))
+                 (bstrchrp(strlist->entry[i], 'M', 0) == 0) ||
+                 (bstrchrp(strlist->entry[i], 'D', 0) == 0))
         {
             if (bstrchrp(strlist->entry[i], ':', 0) != BSTR_ERR)
             {
