@@ -5,14 +5,14 @@
  *
  *      Description:  Header File affinity Module
  *
- *      Version:   5.1.0
- *      Released:  20.11.2020
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Gruber (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -38,6 +38,7 @@ extern int *socket_lock;
 extern int *core_lock;
 extern int *tile_lock;
 extern int *numa_lock;
+extern int *die_lock;
 extern int *sharedl2_lock;
 extern int *sharedl3_lock;
 
@@ -46,6 +47,7 @@ extern AffinityDomains affinityDomains;
 extern int *affinity_thread2core_lookup;
 extern int *affinity_thread2socket_lookup;
 extern int *affinity_thread2numa_lookup;
+extern int *affinity_thread2die_lookup;
 extern int *affinity_thread2sharedl3_lookup;
 extern int affinity_processGetProcessorId();
 extern int affinity_threadGetProcessorId();
