@@ -5,14 +5,14 @@
  *
  *      Description:  Central error handling macros
  *
- *      Version:   5.1.0
- *      Released:  20.11.2020
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Gruber (tr), thomas.roehl@gmail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -118,6 +118,8 @@
 #define CHECK_MSR_READ_ERROR(func) CHECK_AND_RETURN_ERROR(func, MSR read operation failed);
 #define CHECK_PCI_WRITE_ERROR(func) CHECK_AND_RETURN_ERROR(func, PCI write operation failed);
 #define CHECK_PCI_READ_ERROR(func) CHECK_AND_RETURN_ERROR(func, PCI read operation failed);
+#define CHECK_MMIO_WRITE_ERROR(func) CHECK_AND_RETURN_ERROR(func, MMIO write operation failed);
+#define CHECK_MMIO_READ_ERROR(func) CHECK_AND_RETURN_ERROR(func, MMIO read operation failed);
 #define CHECK_POWER_READ_ERROR(func) CHECK_AND_RETURN_ERROR(func, Power register read operation failed);
 #define CHECK_TEMP_READ_ERROR(func) CHECK_AND_RETURN_ERROR(func, Temperature register read operation failed);
 
