@@ -5,13 +5,13 @@
  *
  *      Description:  Interface to hwloc for PCI device lookup
  *
- *      Version:   5.1.0
- *      Released:  20.11.2020
+ *      Version:   5.2.1
+ *      Released:  03.12.2021
  *
  *      Author:   Thomas Gruber (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2020 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2021 NHR@FAU, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -59,26 +59,32 @@ hwloc_pci_init(uint16_t testDevice, char** socket_bus, int* nrSockets)
 
     if (!hwloc_topology)
     {
-        likwid_hwloc_topology_init(&hwloc_topology);
+        LIKWID_HWLOC_NAME(topology_init)(&hwloc_topology);
 #if HWLOC_API_VERSION > 0x00020000
-        likwid_hwloc_topology_set_flags(hwloc_topology, HWLOC_TOPOLOGY_FLAG_WHOLE_SYSTEM );
+        LIKWID_HWLOC_NAME(topology_set_flags)(hwloc_topology, HWLOC_TOPOLOGY_FLAG_WHOLE_SYSTEM );
 #else
-        likwid_hwloc_topology_set_flags(hwloc_topology, HWLOC_TOPOLOGY_FLAG_WHOLE_SYSTEM|HWLOC_TOPOLOGY_FLAG_WHOLE_IO );
+        LIKWID_HWLOC_NAME(topology_set_flags)(hwloc_topology, HWLOC_TOPOLOGY_FLAG_WHOLE_SYSTEM|HWLOC_TOPOLOGY_FLAG_WHOLE_IO );
 #endif
-        likwid_hwloc_topology_load(hwloc_topology);
+        LIKWID_HWLOC_NAME(topology_load)(hwloc_topology);
     }
 
-    for(i = 0; i < likwid_hwloc_get_nbobjs_by_type(hwloc_topology, HWLOC_OBJ_PCI_DEVICE); i++)
+    for(i = 0; i < LIKWID_HWLOC_NAME(get_nbobjs_by_type)(hwloc_topology, HWLOC_OBJ_PCI_DEVICE); i++)
     {
-        obj = likwid_hwloc_get_obj_by_type(hwloc_topology, HWLOC_OBJ_PCI_DEVICE, i);
+        obj = LIKWID_HWLOC_NAME(get_obj_by_type)(hwloc_topology, HWLOC_OBJ_PCI_DEVICE, i);
         if (obj->attr->pcidev.vendor_id != testVendor)
         {
             continue;
         }
         if ((obj->attr->pcidev.vendor_id == testVendor) && (obj->attr->pcidev.device_id == testDevice))
         {
-            socket_bus[cntr] = (char*)malloc(4);
-            sprintf(socket_bus[cntr++], "%02x/", obj->attr->pcidev.bus);
+            hwloc_obj_t walk = obj->parent;
+            while (walk->type != HWLOC_OBJ_SOCKET) walk = walk->parent;
+            if (socket_bus[walk->os_index] == NULL)
+            {
+                socket_bus[walk->os_index] = (char*)malloc(5);
+                snprintf(socket_bus[walk->os_index], 4, "%02x/", obj->attr->pcidev.bus);
+                cntr++;
+            }
         }
     }
     *nrSockets = cntr;
